<?php

$released_versions = [ 100, 110, 111, 112 ];
define( "version", max( $released_versions ) );

// Error reporting
ini_set( "memory_limit", "-1" );
set_time_limit( 0 );
error_reporting( E_ALL );
ini_set( "display_errors", "on" );
ini_set( "xdebug.var_display_max_children", "-1" );
ini_set( "xdebug.var_display_max_data", "-1" );
ini_set( "xdebug.var_display_max_depth", "-1" );

// Define paths
$iroot = realpath( dirname(__FILE__) );
$root  = realpath( dirname( $iroot ) );
define( "iroot", $iroot );
define( "root", $root );
define( "app_root", realpath( root . "/app" ) );
define( "app_core_root", realpath( root . "/app/core" ) );
define( "themes_root", realpath( root . "/themes" ) );

// Required app files
require_once( realpath( app_root . "/config.php" ) );
require_once( realpath( app_core_root . "/class_loader.php" ) );

// Check config
if ( empty( web_addr ) || empty( purchase_code ) || empty( client_code ) || empty( sign_code ) || empty( db_host ) || empty( db_user ) || empty( db_pass ) || empty( db_name ) )
die('install first');

// Initialize app
$loader = new loader();
$loader->_require_core_files( "db" );
$loader->_load_admin_setting( true );
define( "version_installed", $loader->admin->get_setting( "version", 100 ) );
if ( version_installed >= version  )
die('updated already');

foreach( $released_versions as $_a_released_version )
$versions[ $_a_released_version > version_installed ? "no" : "ye" ][] = $_a_released_version;
define( "version_update_to", min( $versions["no"] ) );

// Do the update
if ( !empty( $_POST["update"] ) ){

  $_target_version = version_update_to;
  $_source_sql = iroot . "/__sources/updates/v{$_target_version}.sql";
  if ( file_exists( $_source_sql ) ){

    $_source_sql = realpath( $_source_sql );
    $_source_sql_lines = explode( PHP_EOL, str_replace( [ "\n", "\r\n" ], PHP_EOL, file_get_contents( $_source_sql ) ) );

    $_source_query_lines = [];
		$__query_line = "";
    foreach( $_source_sql_lines as $_source_sql_line ){

			$_source_sql_line = trim( $_source_sql_line );
			if ( empty( $_source_sql_line ) ) continue;
			if ( strlen( $_source_sql_line ) <= 3 ) continue;
			if ( substr( $_source_sql_line, 0, 2 ) == "--" ) continue;

			// is this line a full query?
			$__query_line .= $_source_sql_line;
			if ( substr( $_source_sql_line, -1 ) == ";" ){
				$_source_query_lines[] = substr( $__query_line, 0, -1 );
				$__query_line = "";
			}

		}

    if ( !empty( $_source_query_lines ) ){
      foreach( $_source_query_lines as $_source_query_line ){
        $loader->db->query( $_source_query_line );
      }
    }

  }

  $loader->admin->save_setting( "version", $_target_version );
  header("location: " . web_addr . "/installer/update.php?updated_to=" . $_target_version);
  die;

}

?><!doctype html>
<html>
<head>

  <meta charset="utf-8">
  <title>DigiMuse Updater</title>

  <link href='../themes/__default/assets/third/bootstrap-4.6.0-dist/css/bootstrap.min.css' rel='stylesheet' media='all' type='text/css'>
  <link href='__sources/style.css' rel='stylesheet' media='all' type='text/css'>

</head>
<body class="updater">
  <div class="form_wrapper">
	<form method="post">

    Read documentation before proceeding<br>
		Latest version: <b><?php echo sprintf( "%.2f", version/100 ); ?></b>. Installed version: <b><?php echo sprintf( "%.2f", version_installed/100 ); ?></b>

    <div class="features">
      <?php foreach( $released_versions as $_a_released_version ) : ?>
      <div class="feature update">
        <div class="extension_name">Version <?php echo sprintf( "%.2f", $_a_released_version/100 ); ?></div>
        <div class="extension_status <?php echo in_array( $_a_released_version, $versions["ye"] ) ? "green" : "red"; ?>"><?php echo in_array( $_a_released_version, $versions["ye"] ) ? "Installed" : "Not Installed"; ?></div>
      </div>
      <?php endforeach; ?>
    </div>

		<div class="button_wrapper"><input name="update" type="submit" value="Update from <?php echo sprintf( "%.2f", version_installed/100 ); ?> to <?php echo sprintf( "%.2f", version_update_to/100 ); ?>" class="btn btn-primary"></div>

	</form>
  </div>
</body>
</html>
